@@ -27,6 +27,7 @@
 #include <spice/macros.h>
 #include <spice/vd_agent.h>
 #include <spice/protocol.h>
+#include <stdbool.h>
 
 #include "common/marshaller.h"
 #include "common/messages.h"
@@ -36,7 +37,9 @@
 #include "spice.h"
 #include "red_common.h"
 #include "reds.h"
+#include "reds_stream.h"
 #include "red_channel.h"
+#include "main_channel.h"
 #include "inputs_channel.h"
 #include "migration_protocol.h"
 
@@ -52,7 +55,11 @@
     (4096 + (REDS_AGENT_WINDOW_SIZE + REDS_NUM_INTERNAL_AGENT_MESSAGES) * SPICE_AGENT_MAX_DATA_SIZE)
 
 struct SpiceKbdState {
-    int dummy;
+    bool push_ext;
+
+    /* track key press state */
+    bool key[0x7f];
+    bool key_ext[0x7f];
 };
 
 struct SpiceMouseState {
@@ -220,6 +227,16 @@ static void kbd_push_scan(SpiceKbdInstance *sin, uint8_t scan)
         return;
     }
     sif = SPICE_CONTAINEROF(sin->base.sif, SpiceKbdInterface, base);
+
+    /* track XT scan code set 1 key state */
+    if (scan == 0xe0) {
+        sin->st->push_ext = TRUE;
+    } else {
+        bool *state = sin->st->push_ext ? sin->st->key : sin->st->key_ext;
+        sin->st->push_ext = FALSE;
+        state[scan & 0x7f] = !(scan & 0x80);
+    }
+
     sif->push_scan_freg(sin, scan);
 }
 
@@ -463,14 +480,27 @@ static int inputs_channel_handle_parsed(RedChannelClient *rcc, uint32_t size, ui
     return TRUE;
 }
 
-static void inputs_relase_keys(void)
+static void inputs_release_keys(void)
 {
-    kbd_push_scan(keyboard, 0x2a | 0x80); //LSHIFT
-    kbd_push_scan(keyboard, 0x36 | 0x80); //RSHIFT
-    kbd_push_scan(keyboard, 0xe0); kbd_push_scan(keyboard, 0x1d | 0x80); //RCTRL
-    kbd_push_scan(keyboard, 0x1d | 0x80); //LCTRL
-    kbd_push_scan(keyboard, 0xe0); kbd_push_scan(keyboard, 0x38 | 0x80); //RALT
-    kbd_push_scan(keyboard, 0x38 | 0x80); //LALT
+    int i;
+    SpiceKbdState *st = keyboard->st;
+
+    for (i = 0; i < SPICE_N_ELEMENTS(st->key); i++) {
+        if (!st->key[i])
+            continue;
+
+        st->key[i] = FALSE;
+        kbd_push_scan(keyboard, i | 0x80);
+    }
+
+    for (i = 0; i < SPICE_N_ELEMENTS(st->key_ext); i++) {
+        if (!st->key_ext[i])
+            continue;
+
+        st->key_ext[i] = FALSE;
+        kbd_push_scan(keyboard, 0xe0);
+        kbd_push_scan(keyboard, i | 0x80);
+    }
 }
 
 static void inputs_channel_on_disconnect(RedChannelClient *rcc)
@@ -478,7 +508,7 @@ static void inputs_channel_on_disconnect(RedChannelClient *rcc)
     if (!rcc) {
         return;
     }
-    inputs_relase_keys();
+    inputs_release_keys();
 }
 
 static void inputs_pipe_add_init(RedChannelClient *rcc)
@@ -521,11 +551,17 @@ static void inputs_connect(RedChannel *channel, RedClient *client,
     spice_assert(g_inputs_channel);
     spice_assert(channel == &g_inputs_channel->base);
 
+    if (!reds_stream_is_ssl(stream) && !red_client_during_migrate_at_target(client)) {
+        main_channel_client_push_notify(red_client_get_main(client),
+                                        "keyboard channel is insecure");
+    }
+
     spice_printerr("inputs channel client create");
     icc = (InputsChannelClient*)red_channel_client_create(sizeof(InputsChannelClient),
                                                           channel,
                                                           client,
                                                           stream,
+                                                          FALSE,
                                                           num_common_caps, common_caps,
                                                           num_caps, caps);
     if (!icc) {
