@@ -39,8 +39,10 @@
 #endif // USE_OPENGL
 #include "reds.h"
 #include "dispatcher.h"
-#include "red_dispatcher.h"
 #include "red_parse_qxl.h"
+#include "spice_server_utils.h"
+
+#include "red_dispatcher.h"
 
 static int num_active_workers = 0;
 
@@ -1070,7 +1072,7 @@ static RedChannel *red_dispatcher_cursor_channel_create(RedDispatcher *dispatche
     return cursor_channel;
 }
 
-RedDispatcher *red_dispatcher_init(QXLInstance *qxl)
+void red_dispatcher_init(QXLInstance *qxl)
 {
     RedDispatcher *red_dispatcher;
     RedWorkerMessage message;
@@ -1083,6 +1085,8 @@ RedDispatcher *red_dispatcher_init(QXLInstance *qxl)
     sigset_t curr_sig_mask;
     ClientCbs client_cbs = { NULL, };
 
+    spice_return_if_fail(qxl->st->dispatcher == NULL);
+
     quic_init();
     sw_canvas_init();
 #ifdef USE_OPENGL
@@ -1173,12 +1177,12 @@ RedDispatcher *red_dispatcher_init(QXLInstance *qxl)
         reds_register_channel(cursor_channel);
     }
 
-    qxl->st->qif->attache_worker(qxl, &red_dispatcher->base);
-    qxl->st->qif->set_compression_level(qxl, calc_compression_level());
-
+    qxl->st->dispatcher = red_dispatcher;
     red_dispatcher->next = dispatchers;
     dispatchers = red_dispatcher;
-    return red_dispatcher;
+
+    qxl->st->qif->attache_worker(qxl, &red_dispatcher->base);
+    qxl->st->qif->set_compression_level(qxl, calc_compression_level());
 }
 
 struct Dispatcher *red_dispatcher_get_dispatcher(RedDispatcher *red_dispatcher)
