@@ -22,24 +22,6 @@
 #include <errno.h>
 #include "red_common.h"
 
-
-static inline void set_bit(int index, uint32_t *addr)
-{
-    uint32_t mask = 1 << index;
-    __sync_or_and_fetch(addr, mask);
-}
-
-static inline void clear_bit(int index, uint32_t *addr)
-{
-    uint32_t mask = ~(1 << index);
-    __sync_and_and_fetch(addr, mask);
-}
-
-static inline int test_bit(int index, uint32_t val)
-{
-    return val & (1u << index);
-}
-
 enum {
     RED_WORKER_PENDING_WAKEUP,
     RED_WORKER_PENDING_OOM,
@@ -86,6 +68,7 @@ enum {
     RED_WORKER_MESSAGE_CURSOR_CHANNEL_CREATE,
 
     RED_WORKER_MESSAGE_MONITORS_CONFIG_ASYNC,
+    RED_WORKER_MESSAGE_DRIVER_UNLOAD,
 
     RED_WORKER_MESSAGE_COUNT // LAST
 };
